use std::{path::PathBuf, process::exit};

use clap::{Parser, Subcommand};
use color_eyre::eyre::Result;
use tracing::info;

use crate::{commands, State};

#[derive(Parser, Debug, Clone, PartialEq, Eq)]
#[command(version, about, long_about = None)]
pub struct Cli {
    #[arg(short, long)]
    pub state: Option<PathBuf>,
    #[command(subcommand)]
    pub command: Option<Commands>,
}

#[derive(Debug, Clone, Subcommand, PartialEq, Eq)]
pub enum Commands {
    Print { tag: Option<String> },
    Tag { tag: String, objects: Vec<String> },
    Remove { tag: String, object: String },
    Init,
    Index,
}

pub fn run(flags: &Cli) -> Result<()> {
    let command = flags.command.clone().unwrap();
    if command == Commands::Init {
        info!("Creating empty .state.json");
        commands::init()?;
        exit(0)
    }

    let mut state = State::get_state(&flags)?;

    match &command {
        Commands::Print { tag } => commands::print(&state, &tag),
        Commands::Tag { tag, objects } => commands::tag(&mut state, tag, objects),
        Commands::Remove { tag, object } => commands::remove(&mut state, tag, object)?,
        Commands::Init => exit(1),
        Commands::Index => commands::index(&mut state)?,
    }

    state.write_state(&flags)?;
    Ok(())
}
