use std::{collections::HashSet, path::PathBuf, str::FromStr};

use color_eyre::eyre::Result;

use crate::State;

pub fn index(state: &mut State) -> Result<()> {
    for file in index_recursion(&state, PathBuf::from_str(".")?) {
        if state.tags.get(&file).is_some() {
            continue;
        }
        state.tags.insert(file, HashSet::new());
    }
    Ok(())
}

fn index_recursion(state: &State, current_path: PathBuf) -> Vec<String> {
    let mut paths = Vec::new();
    let is_file = std::fs::metadata(&current_path).unwrap().is_file();
    if is_file {
        return vec![current_path.to_string_lossy().to_string()];
    }
    for path in std::fs::read_dir(&current_path).unwrap() {
        paths.append(&mut index_recursion(state, path.unwrap().path()));
    }
    paths
}
