use std::{
    collections::{HashMap, HashSet},
    path::PathBuf,
};

use color_eyre::eyre::Result;
use serde::{Deserialize, Serialize};

use crate::cli::Cli;

#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
pub struct State {
    pub tags: HashMap<String, HashSet<String>>,
}

impl State {
    pub fn get_state(flags: &Cli) -> Result<Self> {
        let statefile_location = match &flags.state {
            Some(v) => v.to_owned(),
            None => {
                let mut current_dir = PathBuf::new();
                loop {
                    let mut testing_path = current_dir.clone();
                    testing_path.push(".state.json");
                    if testing_path.try_exists()? {
                        break;
                    }
                    current_dir.push("..");
                }
                current_dir.push(".state.json");
                current_dir
            }
        };
        let state = std::fs::read_to_string(statefile_location).unwrap();
        let state: State = serde_json::from_str(&state)?;
        Ok(state)
    }
    pub fn write_state(&self, flags: &Cli) -> Result<()> {
        let state = match &flags.state {
            Some(v) => v.to_owned(),
            None => {
                let mut current_dir = PathBuf::new();
                loop {
                    let mut testing_path = current_dir.clone();
                    testing_path.push(".state.json");
                    if testing_path.try_exists()? {
                        break;
                    }
                    current_dir.push("..");
                }
                current_dir.push(".state.json");
                current_dir
            }
        };

        std::fs::write(state, serde_json::to_string_pretty(self)?)?;

        Ok(())
    }
}
